/*****************************************************************************************
 This program is a Time of Day Clock that communicates with the PINK(SB70) embedded web server.
 Runs on 'SX Tech Board' with SX28AC and 50MHz resonator.
 Add your own Switches connected to RB0-RB2 for time setting,
 Parallax 2x16 Serial LCD connected to RC.7, RA.2 from PINK and RA.3 to PINK.
 

 Author: M. Gebauer 
 Version 1.0
 Date: 13 Nov 2005
 
 Required hardware: SX tech developement board, Parallax 2x16 Serial LCD, PINK(SB70)
 Required files:main.cpp, sx_config.h, VP_Lib.h
 compiler: C2C++ from picant.com - http://picant.com/c2cpp/cpp.html
                   
 Description:
Time of Day clock with web interface and email. 
1) Set time from webpage
2) Set time from switches.
	SW 1 on RB.0 selects Hour,Minute,AM/PM.
	SW 2 on RB.1 Increments display
	SW 3 on RB.2 Decrements display

3) Email SX clock web page

 *****************************************************************************************/
#include "system.h"
#include "sx_config.h"
#include "VP_Lib.h"

/*****************************************************************************************
 defined varibles
 *****************************************************************************************/
char sw_state=0;								// holds debounce state 
char mode_state=0;								// hold clock adj. mode state
char lastSecond=255;								// holds value of previous second
char lastMinute=255;								// holds value of previous minute
char lastHour=255;								// holds value of previous hour

/*****************************************************************************************
 defined constants
 *****************************************************************************************/
#define STNetworkRdy_mask 1
#define STvarPost_mask 2
#define STEmailNotRDY_mask 4
#define Rx1Rdy_mask 8
/*****************************************************************************************
 Uncomment the following to enable VP multitask switcher
 *****************************************************************************************/

#define multitasker

#ifdef multitasker
char isrMultiplex=0;							// Multitasker thread select counter
#endif

/*****************************************************************************************
  Set Timer 
  ISR (RTCC = intPeriod-no of instructions executed in the ISR)
 *****************************************************************************************/
#ifndef intPeriod								// check if intPeriod has beed defined. if not,
#define intPeriod 217
#endif


void interrupt( void )
{
    //Use this trick to get 2 free instructions
    //to debug this code with SX-KEY from Parallax
    goto skip2words_label;
    asm ORG 3
    skip2words_label:
    
	/***** Your ISR goes here *****/
//	VP_pwm_adc();

#ifdef multitasker
	/*****************************************************************************************
	 --------------------------------------------VP:VP Multitasker----------------------------
	 Virtual Peripheral Multitasker :
	 First thread running at the (interrupt rate/2) and the following
	 8 individual threads, each running at the (interrupt rate/16).
	 Change them below:
	 Input variable(s): isrmultiplex: variable used to choose threads
	 Output variable(s): None,executes the next thread
	 Variable(s) affected: isrmultiplex
	 Flag(s) affected: None
	 *****************************************************************************************/

		asm bank	_isrMultiplex	
		asm mov 	w,_isrMultiplex
		++isrMultiplex;
		asm add 	pc,w
		goto	isrThread1;
		goto	isrThread2;
		goto	isrThread1;
		goto	isrThread3;
		goto	isrThread1;
		goto	isrThread2;
		goto	isrThread1;
		goto	isrThread4;
		goto	isrThread1;
		goto	isrThread2;
		goto	isrThread1;
		goto	isrThread5;
		goto	isrThread1;
		goto	isrThread2;
		goto	isrThread1;
		isrMultiplex=0;
		goto	isrMultitaskOut;
	
/*********************************************************************************************
 VP: VP Multitasker
  ISR TASKS
 *********************************************************************************************/

	isrThread1:														// Serviced at ISR rate/2
	/*****************************************************************************************
	Virtual Peripheral: 

		Input variable(s): 
		Output variable(s): 
		Variable(s) affected: 
		Flag(s) affected: 
	 *****************************************************************************************/

	/***** Your VP goes here *****/
	VP_UART1();

	//----------------------------------------------------------------------------------------
	goto		isrMultitaskOut;				// 7 cycles until mainline program resumes execution
	//----------------------------------------------------------------------------------------

	isrThread2:															// Serviced at ISR rate/4
	/*****************************************************************************************
	Virtual Peripheral: 

		Input variable(s): 
		Output variable(s): 
		Variable(s) affected: 
		Flag(s) affected: 
	 *****************************************************************************************/

	/***** Your VP goes here *****/
	VP_UART();

	//----------------------------------------------------------------------------------------
	goto		isrMultitaskOut;				// 7 cycles until mainline program resumes execution
	//----------------------------------------------------------------------------------------

	isrThread3:														// Serviced at ISR rate/16
	/*****************************************************************************************
	Virtual Peripheral: 

		Input variable(s): 
		Output variable(s): 
		Variable(s) affected: 
		Flag(s) affected: 
	 *****************************************************************************************/

	/***** Your VP goes here *****/
	VP_clock();

	//----------------------------------------------------------------------------------------
	goto		isrMultitaskOut;		// 7 cycles until mainline program resumes execution
	//----------------------------------------------------------------------------------------

	isrThread4:														// Serviced at ISR rate/16
	/*****************************************************************************************
	Virtual Peripheral: 

		Input variable(s): 
		Output variable(s): 
		Variable(s) affected: 
		Flag(s) affected: 
	 *****************************************************************************************/

	/***** Your VP goes here *****/

	//----------------------------------------------------------------------------------------
	goto		isrMultitaskOut;		// 7 cycles until mainline program resumes execution
	//----------------------------------------------------------------------------------------

	isrThread5:														// Serviced at ISR rate/16
	/*****************************************************************************************
	Virtual Peripheral: 

		Input variable(s): 
		Output variable(s): 
		Variable(s) affected: 
		Flag(s) affected: 
	 *****************************************************************************************/

	/***** Your VP goes here *****/

	//----------------------------------------------------------------------------------------
	goto		isrMultitaskOut;		// 7 cycles until mainline program resumes execution
	//----------------------------------------------------------------------------------------

	isrMultitaskOut:
#endif
/*********************************************************************************************
 -------------------------------------- End of Multitasker -----------------------------------
 *********************************************************************************************/

	/*****************************************************************************************
	 Set RTCC on return to execute every intPeriod cycles
	 *****************************************************************************************/

	asm	mov	w,#-intPeriod
	asm retiw
}

/*****************************************************************************************
	Sub-routines
 *****************************************************************************************/

char getPINKChar( void )
{
	while((flags & Rx1Rdy_mask) != Rx1Rdy_mask );				// wait for response
	clear_bit(flags,Rx1Rdy);
	return Rx1byte;
}

void printfPINK( const char* text )
{
    char i = 0;
    while( text[i] != 0 ) sendbyte1( text[i++] ); 
    sendbyte1(0);
}

char getPINKstrToInt( void )									// waits for string from PINK and converts it to Integer
{
char tmp1=0;
char tmp2=0;
char i=0;
do_loop:
	tmp1=getPINKChar();											// get char from PINK.
	if (tmp1 == 0) goto exit_do;								// exit if end of string							
	if(tmp1 >= 48) tmp1=tmp1-48;								// convert ascii to integer
	if(tmp1 == 32) tmp1=0;										// replace space with 0
	if(i==0) tmp1=tmp1*10;										// if tens value then multiply by 10
	tmp2=tmp2+tmp1;												// accumulate integer in register
	i++;
	goto do_loop;
exit_do:
	if( i==1 ) tmp2=tmp2/10;									// if number less than 2 digits move tens back to ones
	return tmp2;
}

void savePINKdata( void )
{
char i=0;
char tmp;
	while(i < 4)
	{
		printf1("!NB0R");										// select PINK var to read
		sendbyte1((i)/10+48);
		sendbyte1((i)%10+48);
		tmp=getPINKstrToInt();									// get integer from PINK.
		//store ineger value from pink in proper register
		switch (i)
		{
		case 0:
			hours=tmp;
			break;
		case 1:
			minutes=tmp;
			break;
		case 2:
			seconds=tmp;
			break;
		case 3:
			if (tmp == ((65-48)*10+77-48))
			{
				if(hours == 12) hours=0;						// adjust hour count AM
				printfPINK("!NB0W03:AM");						// update web
			}
			else
			{
				if(hours < 12) hours=hours+12;					// adjust hour count PM
				printfPINK("!NB0W03:PM");						// update web
			}				
			break;
		}
		++i;
	}		
}

void waitIfMailSending( void )
{
	printf1("!NB0ST");											// get status from PINK
		while( (getPINKChar() & STEmailNotRDY_mask) == STEmailNotRDY_mask ) 
	printf1("!NB0ST");											// wait for mail to be sent
}

void checkPINK( void )
{
	printf1("!NB0ST");										// get status from PINK
	if((getPINKChar() & STvarPost_mask) == STvarPost_mask )	// check if post from PINK
	{
	delay_ms(100);											// wait for all PINK vars to post.
		printf1("!NB0SV");									// request number of last var posted
		if( getPINKChar()!=4 )
		{
				savePINKdata();								// get PINK post data
		}else{
				waitIfMailSending();						// wait if busy sending mail
				printf1("!NB0SM");							// send mail
				sendbyte1(0);								// !don't know why but this works!
		}
	}
}

void dataToPINK( char varaddr, char data, char mask )
{
	printf1("!NB0W");
	sendbyte1(varaddr/10+48);									// send var address tens
	sendbyte1(varaddr%10+48);									// send var address ones
	printf1(":");
	
	if(((data/10) == 0) & mask) 								// if result = 0 and mask is TRUE
	{
		printf1(" ");											// send a space
	}
	else
	{
		sendbyte1(data/10+48);									// else send tens value with ascii offset
	}
	sendbyte1(data%10+48);
	sendbyte1(0);
}

void datatoLCD( char data, char mask )
{
	if(((data%100/10) == 0) & mask) 							// if result = 0 and mask is TRUE
	{
		printf(" ");											// send a space
	}
	else
	{
		sendbyte(data%100/10+48);								// else send tens value with ascii offset
	}
	sendbyte(data%10+48);										// send ones value with ascii offset
}

void displayTime ( void )
{
	sendbyte(152);												// place cursor on line 1 pos. 5

	if( (milliseconds > 500) & (mode_state == 1) & !sw_state )	// if mode = hours adj. - blink hours
	{
		printf("  ");
	}		
	else
	{
		checkPINK();											// if Post, get it
		if ( hours <= 12)										// adjust for 12 hour clock
		{
			if( hours == 0 )
			{
				datatoLCD( 12,TRUE );							// output data byte to LCD
				dataToPINK( 0, 12, TRUE );						// output data to PINK
			}
			else
			{
				datatoLCD( hours,TRUE );						// output data byte to LCD
				dataToPINK( 0, hours, TRUE );					// output data to PINK
			}
		}
		else
		{
			datatoLCD( hours-12, TRUE );						// output data byte to LCD
			dataToPINK( 0, hours-12, TRUE );					// output data to PINK
		}
	}
	if( (milliseconds > 500) & (mode_state == 0) )				// if mode = clock - blink :
	{
		printf(" ");
	}
	else
	{
		printf(":");
	}
	
	if( (milliseconds > 500) & (mode_state == 2) & !sw_state )	// if mode = minutes adj. - blink minutes
	{
		printf("  ");
	}		
	else
	{
		checkPINK();											// if Post, get it
		datatoLCD( minutes, FALSE );							// output data byte to LCD
		dataToPINK( 1, minutes, FALSE );						// output data to PINK
	}

	checkPINK();												// if Post, get it
	dataToPINK( 2, seconds, FALSE );							// output data to PINK
	lastSecond=seconds;											// store for next test

	if( (milliseconds > 500) & (mode_state == 3) & !sw_state ) 	// if mode = AM/PM adj. - blink AM/PM
	{
		printf("   ");
	}		
	else
	{
		checkPINK();											// if Post, get it
		if (hours < 12)											// using 12 hour clock display AM or PM
		{
			printf(" AM");
			printfPINK("!NB0W03:AM");
		}
		else
		{
			printf(" PM");
			printfPINK("!NB0W03:PM");		
		}
	}
	if(sec_timer == 0) mode_state=0;							// check for time out on clock adjust mode
 }

void sw_actions( void )
{
	sec_timer=60;												// set time adjust timeout period in seconds

	switch ((~RB) & 111b)
	{
	case 001b:													// SW 1 -scroll through hours/minutes/AM/PM time adjust modes
				mode_state=(++mode_state & 11b);				//scrolls through modes (0=normal, 1= hours adj., 2=min adj., 3= AM/PM adj.)
				seconds=0;										// zero seconds
				milliseconds=0;									// zero milliseconds
				break;
	case 010b:													// SW 2 - ****** increment time ******
				switch (mode_state) 
				{
				case 1:											// increment hours
						if(hours > 12)
						{
							++hours;
							if(hours == 24) hours=12;
						}
						else
						{
							++hours;
							if(hours == 12) hours=0;
						}
						break;
				case 2:											// increment minutes
						++minutes;
						if(minutes == 60) minutes=0;
						break;
				case 3:											// toggle AM/PM
						if(hours >= 12) hours=hours-12;
						else hours=hours+12;
						break;
				}
				break;
	case 100b:													// SW 3 - ****** decrement time ******
				switch (mode_state) 
				{
				case 1:											// decrement hours
						if(hours < 12)
						{
							--hours;
							if(hours == 255) hours=11;
						}
						else
						{
							--hours;
							if(hours == 11) hours=23;
						}
						break;
				case 2:											// decrement minutes
						--minutes;
						if(minutes == 255) minutes=59;
						break;
				case 3:											// toggle AM/PM
						if(hours >= 12) hours=hours-12;
						else hours=hours+12;
						break;
				}
				break;
	}
}

void get_switches( void )
{
  switch (sw_state)					// ******* switch debounce and repeat. calls switch action routine. *******
    {
      case 0: // key open
        if ((~RB) & 111b)
          {
            sw_state=1;
            ms_timer=50; 										// milliseconds (this is a downcounter)
          }
        break;

      case 1: 													// wait for debounce delay
        if (!((~RB) & 111b))
          {
            sw_state=0;
            break;
          }
        if (ms_timer == 0) 										// debounce timer expired
          {
			sw_actions();
            ms_timer=250; 										// milliseconds
            sw_state=2;
          }
        break;

      case 2: 													// wait to repeat the key action
        if (!((~RB) & 111b))
          {
            sw_state=0;
            break;
          }
        if (ms_timer == 0) 										// timer expired, increment again
          {
			sw_actions();
            ms_timer=75; 										// milliseconds
            // stay in this state
          }
        break;
    }
}

void init( void )
{
	delay_s(2);													// allow PINK to initialize
	sendbyte(12);												// clear LCD
	delay_ms(5);												// manditory after LCD clear
	sendbyte(128);												// position cursor at line 0, col 0
	sendbyte(22);												// turn LCD cursor off
	sendbyte(17);												// turn LCD backlight on
	printf("    SX28 time");									// LCD display heading
}

/*****************************************************************************************
	end Sub-routines
 *****************************************************************************************/

void main( void )
{
char tmp=0;
	sx_config();												// initialize port and RTCC
	init();														// setup start parameters

    //Start endless loop
    while( 1 )
    {
		displayTime();											// display time every 10ms
		get_switches();											// check switches
	}
}